import React, {
    memo,
    useState,
    useEffect,
    useMemo,
    useCallback,
    useRef,
} from "https://esm.sh/react@18.2.0";

import ReactDOM from "https://esm.sh/react-dom@18.2.0";
import STORAGE_DATA from "../../gtgorgtw/js/constant.js?v=1.0";

const CardFrame = memo(({ config, currentQuestionIndex, imgType }) => {
    if (
        !config ||
        !config.cardStart ||
        !config.realBackgroundCloud ||
        !config.cardQuestion
    ) {
        return null;
    }

    const backgroundImages = {
        start: config.cardStart,
        cloud: config.realBackgroundCloud,
        question: config.cardQuestion,
    };

    return (
        <div className="relative h-full w-full max-w-[430px] rounded-[15px] overflow-hidden">
            <div className="relative w-full rounded-[15px] min-h-[500px]">
                {Object.entries(backgroundImages).map(([type, src]) => (
                    <img
                        key={type}
                        className={`object-contain w-full transition-opacity duration-300 ${
                            imgType === type
                                ? "opacity-100"
                                : "opacity-0 absolute inset-0"
                        }`}
                        src={src}
                        alt={`background-${type}`}
                        loading={imgType === type ? "eager" : "lazy"}
                    />
                ))}
            </div>
        </div>
    );
});

const StartPage = memo(({ config, onStart, currentQuestionIndex }) => {
    const isMounted = useRef(false);

    useEffect(() => {
        isMounted.current = true;

        return () => {
            isMounted.current = false;
        };
    }, []);

    if (
        !config ||
        !config.realBackgroundCloud ||
        !config.mainVisual ||
        !config.startHint ||
        !config.buttons ||
        !config.buttons.startButton
    ) {
        return (
            <div className="flex w-full min-h-[100dvh] items-center justify-center">
                <p>載入中...</p>
            </div>
        );
    }

    return (
        <div className="flex w-full min-h-[100dvh] items-center justify-center max-w-[430px] rounded-[15px]">
            <div
                style={{
                    backgroundImage: `url(${config.realBackgroundCloud})`,
                    backgroundSize: "cover",
                    backgroundPosition: "center",
                    backgroundRepeat: "no-repeat",
                }}
                className="rounded-[15px] relative w-full transition-opacity duration-300 opacity-100 "
            >
                <CardFrame
                    imgType="start"
                    config={config}
                    currentQuestionIndex={currentQuestionIndex}
                />
                <div
                    className="absolute bottom-[120px] left-1/2 transform -translate-x-1/2 "
                    onClick={onStart}
                >
                    <img
                        className="start-animation-visual-width"
                        src={config.mainVisual}
                        alt="start-animation-visual"
                        loading="eager"
                    />
                </div>

                <button
                    className="start-button-float absolute bottom-[10px] left-1/2 transform -translate-x-1/2 flex flex-col justify-center items-center"
                    id="start-test"
                    onClick={onStart}
                >
                    <img
                        className="start-hint-width pb-1"
                        src={config.startHint}
                        alt="start-hint-image"
                        loading="eager" //優先
                    />
                    <img
                        className="start-button-width"
                        src={config.buttons.startButton}
                        alt="start-button-image"
                        loading="eager"
                    />
                </button>
            </div>
        </div>
    );
});

const QuestionPage = memo(
    ({ currentQuestion, onAnswer, config, onBack, currentScore }) => {
        const handleButtonClick = async (score, choiceLetter) => {
            const button = document.activeElement;
            if (button) {
                button.blur();
                button.style.pointerEvents = "none";
                setTimeout(() => {
                    button.style.pointerEvents = "auto";
                }, 300);
            }

            if (currentQuestion.id === 10) {
                onAnswer(score, choiceLetter);
            } else {
                onAnswer(score);
            }
        };

        const handleBackClick = () => {
            const button = document.activeElement;
            if (button) {
                button.blur();
                button.style.pointerEvents = "none";
                setTimeout(() => {
                    button.style.pointerEvents = "auto";
                }, 300);
            }
            onBack();
        };

        const getButtonConfig = (questionId) => {
            return config.questionOption[questionId];
        };

        const buttons = getButtonConfig(currentQuestion.id);

        return (
            <div className="flex w-full min-h-[100dvh] items-center justify-center max-w-[430px] rounded-[15px]">
                <div
                    style={{
                        backgroundImage: `url(${config.realBackgroundCloud})`,
                        backgroundSize: "cover",
                        backgroundPosition: "center",
                        backgroundRepeat: "no-repeat",
                    }}
                    className="rounded-[15px] relative w-full transition-opacity duration-300 opacity-100"
                >
                    <CardFrame imgType="question" config={config} />
                    <button
                        onClick={handleBackClick}
                        className="absolute top-[170px] left-[70px] z-10 w-10 h-8 flex items-center justify-center
                                text-2xl font-bold text-[#402529] hover:text-[#5a363b] transition-colors
                                hover:bg-black/5 rounded-full"
                        style={{ fontFamily: "sans-serif" }}
                        id="back-to-former-page"
                    >
                        <img
                            className="back-button-slide w-[28px]"
                            src={config.buttons.backButton}
                            alt="back-button"
                        />
                    </button>
                    <div className="absolute top-[110px] left-1/2 transform -translate-x-1/2 text-center w-full">
                        <div className="relative mb-[60px]">
                            <img
                                className="w-[45px] max-w-[280px] mx-auto"
                                style={{ marginBottom: 28 }}
                                src={config.step[`step${currentQuestion.id}`]}
                                alt={`step${currentQuestion.id}`}
                            />
                        </div>
                        <img
                            className="max-w-[300px] sm:w-[70%] mx-auto mb-[50px]"
                            src={config.questionImage[`Q${currentQuestion.id}`]}
                            alt={`Question_${currentQuestion.id}`}
                        />
                        <div className="flex flex-col gap-[20px] items-center mt-6">
                            {buttons.map((button, index) => (
                                <button
                                    key={index}
                                    onClick={() =>
                                        handleButtonClick(
                                            button.score,
                                            button.letter
                                        )
                                    }
                                    onTouchEnd={(e) => e.target.blur()}
                                    id={`question-option-${index + 1}`}
                                >
                                    <img
                                        className="button-answer-width"
                                        src={button.img}
                                        alt={`option_${index + 1}`}
                                    />
                                </button>
                            ))}
                        </div>
                    </div>
                </div>
            </div>
        );
    }
);

const ResultPage = memo(
    ({
        resultImage,
        totalScore,
        config,
        onRestart,
        question10Choice,
        resultType,
    }) => {
        const [showConfirm, setShowConfirm] = useState(false);
        const [customPadding, setCustomPadding] = useState("16px");
        const [customImagePadding, setCustomImagePadding] = useState("16px");
        const [customWidth, setCustomWidth] = useState("320px");
        const [mergedImageUrl, setMergedImageUrl] = useState("");
        const [isLoading, setIsLoading] = useState(true);
        const [showLotteryButton, setShowLotteryButton] = useState(true);

        const getQuestion10AnswerText = useCallback((choice) => {
            console.log("當前第10題選擇:", choice);

            if (choice === "a") return "中頭獎";
            if (choice === "b") return "升職加薪";
            if (choice === "c") return "擁有一億存款";

            return "";
        }, []);

        const handleShare = async () => {
            try {
                const imageUrl = mergedImageUrl || resultImage;

                if (navigator.share) {
                    const blob = await fetch(imageUrl).then((r) => r.blob());
                    const file = new File([blob], "result.png", {
                        type: "image/png",
                    });

                    await navigator.share({
                        title: "台中廣天宮｜測測你是哪隻小財怪",
                        text: `💰你是財運爆棚還是存款黑洞？😂 測測你是哪種財怪，不論是財氣一級棒的天財，還是月底吃土的蠢財？都能解鎖台中廣天宮專屬招財法寶！💸✨ 👉【立即開測】
https://www.gtg.org.tw/psychological`,
                        files: [file],
                    });
                }
            } catch (error) {
                console.error("Share Failed:", error);

                if (navigator.share) {
                    try {
                        await navigator.share({
                            title: "台中廣天宮｜測測你是哪隻小財怪",
                            text: `💰你是財運爆棚還是存款黑洞？😂 測測你是哪種財怪，不論是財氣一級棒的天財，還是月底吃土的蠢財？都能解鎖台中廣天宮專屬招財法寶！💸✨ 👉【立即開測】
                            https://www.gtg.org.tw/psychological`,
                        });
                    } catch (shareError) {
                        console.error("Text share failed:", shareError);
                    }
                }
            }
        };

        const handleRestartClick = () => {
            setShowConfirm(true);
        };

        const handleConfirmRestart = () => {
            setShowConfirm(false);
            onRestart();
        };

        const handleCloseConfirm = () => {
            setShowConfirm(false);
        };

        const handleDownload = () => {
            try {
                const imageUrl = mergedImageUrl || resultImage;

                if (!imageUrl) {
                    console.error("No image URL available for download");
                    return;
                }

                if (imageUrl.startsWith("data:")) {
                    const byteString = atob(imageUrl.split(",")[1]);
                    const mimeString = imageUrl
                        .split(",")[0]
                        .split(":")[1]
                        .split(";")[0];
                    const ab = new ArrayBuffer(byteString.length);
                    const ia = new Uint8Array(ab);

                    for (let i = 0; i < byteString.length; i++) {
                        ia[i] = byteString.charCodeAt(i);
                    }

                    const blob = new Blob([ab], { type: mimeString });
                    const url = URL.createObjectURL(blob);

                    const link = document.createElement("a");
                    link.href = url;
                    link.download = "財運結果.png";
                    document.body.appendChild(link);
                    link.click();

                    setTimeout(() => {
                        document.body.removeChild(link);
                        URL.revokeObjectURL(url);
                    }, 100);
                } else {
                    fetch(imageUrl)
                        .then((response) => response.blob())
                        .then((blob) => {
                            const url = URL.createObjectURL(blob);
                            const link = document.createElement("a");
                            link.href = url;
                            link.download = "財運結果.png";
                            document.body.appendChild(link);
                            link.click();

                            setTimeout(() => {
                                document.body.removeChild(link);
                                URL.revokeObjectURL(url);
                            }, 100);
                        })
                        .catch((error) => {
                            console.error("Error downloading image:", error);
                        });
                }
            } catch (error) {
                console.error("Download failed:", error);

                if (mergedImageUrl || resultImage) {
                    window.open(mergedImageUrl || resultImage, "_blank");
                }
            }
        };

        useEffect(() => {
            const currentUtcTime = Date.now();
            const targetDate = new Date("2025-04-17T00:00:00+08:00").getTime();

            setShowLotteryButton(currentUtcTime < targetDate);
        }, []);

        useEffect(() => {
            if (!resultImage) return;

            setIsLoading(true);

            const createMergedImage = async () => {
                try {
                    const img = new Image();
                    img.crossOrigin = "anonymous";

                    const targetDate = new Date(
                        "2025-04-17T00:00:00+08:00"
                    ).getTime();
                    const currentUtcTime = Date.now();
                    const showLotteryBtn = currentUtcTime < targetDate;

                    img.onload = () => {
                        const canvas = document.createElement("canvas");
                        canvas.width = img.width;
                        canvas.height = img.height;
                        const ctx = canvas.getContext("2d");

                        ctx.drawImage(img, 0, 0);

                        const answer =
                            getQuestion10AnswerText(question10Choice);

                        if (answer) {
                            const line1 = `如果想要${answer}，`;
                            const line2 = `農曆三月十五 國曆4/12就來台中廣天宮參加財神借庫!`;

                            const fontSize = img.width * 0.03;
                            ctx.font = `${fontSize}px Arial, "Noto Sans TC", sans-serif`;
                            ctx.fillStyle = "#000000";
                            ctx.textAlign = "center";

                            ctx.fillText(
                                line1,
                                img.width * 0.5,
                                img.height * 0.805
                            );

                            ctx.fillText(
                                line2,
                                img.width * 0.5,
                                img.height * 0.805 + fontSize * 1.15
                            );
                        }

                        const mergedImageUrl = canvas.toDataURL("image/png");
                        setMergedImageUrl(mergedImageUrl);
                        setIsLoading(false);
                    };

                    img.onerror = () => {
                        console.error("Error loading image");
                        setIsLoading(false);
                    };

                    img.src = resultImage;
                } catch (error) {
                    console.error("Error creating merged image:", error);
                    setIsLoading(false);
                }
            };

            createMergedImage();
        }, [resultImage, question10Choice, getQuestion10AnswerText]);

        useEffect(() => {
            const handleResize = () => {
                setCustomPadding(
                    window.innerWidth >= 500
                        ? "32px"
                        : window.innerWidth >= 400
                        ? "32px"
                        : window.innerWidth >= 382
                        ? "16px"
                        : window.innerWidth >= 366
                        ? "8px"
                        : 0
                );
            };
            const handleImageResize = () => {
                setCustomImagePadding(
                    window.innerWidth >= 500
                        ? "24px"
                        : window.innerWidth >= 400
                        ? "8px"
                        : window.innerWidth >= 382
                        ? "4px"
                        : window.innerWidth >= 366
                        ? "2px"
                        : 0
                );
            };
            const handleWidth = () => {
                setCustomWidth(window.innerWidth >= 360 ? "350px" : "320px");
            };

            handleResize();
            handleImageResize();
            handleWidth();

            window.addEventListener("resize", handleResize);
            window.addEventListener("resize", handleImageResize);
            window.addEventListener("resize", handleWidth);

            return () => {
                window.removeEventListener("resize", handleResize);
                window.removeEventListener("resize", handleImageResize);
                window.removeEventListener("resize", handleWidth);
            };
        }, []);

        return (
            <div className="w-full flex flex-col items-center">
                <div
                    className="relative w-full max-w-[430px] bg-red "
                    style={{
                        padding: `32px ${customImagePadding} 16px ${customImagePadding}`,
                        backgroundImage: `url(${config.realBackgroundCloud})`,
                        backgroundSize: "cover",
                        backgroundPosition: "center top",
                        backgroundRepeat: "no-repeat",
                        position: "relative",
                        zIndex: 1,
                    }}
                >
                    <div className="outcome-container rounded-[15px] overflow-hidden min-h-[648px] flex justify-center items-center">
                        <div className="relative w-[370px]">
                            <img
                                className="w-full w-[370px] mx-auto object-contain z-10"
                                src={mergedImageUrl || resultImage}
                                alt={resultType || "top-天財"}
                            />

                            <button
                                id={
                                    showLotteryButton
                                        ? "go-to-lottery"
                                        : "call-to-action"
                                }
                                // 抽獎網址置換
                                onClick={() =>
                                    window.open(
                                        `${
                                            showLotteryButton
                                                ? "https://pse.is/7ar4ne"
                                                : "https://pse.is/7ar4ne"
                                        }`
                                    )
                                }
                                className="absolute bottom-[28%] right-[43px] w-[70px] z-20 hover:opacity-90 cursor-pointer"
                                style={{
                                    cursor: "pointer",
                                    pointerEvents: "auto",
                                }}
                            >
                                <img
                                    src={
                                        showLotteryButton
                                            ? config.buttons.lotteryButton
                                            : config.buttons.callToAction
                                    }
                                    alt={
                                        showLotteryButton
                                            ? "lottery-button"
                                            : "know-the-details"
                                    }
                                    className="w-full h-auto object-contain"
                                />
                            </button>
                        </div>
                    </div>
                </div>
                <div
                    className="result-content flex flex-col justify-content items-center max-w-[430px] w-full"
                    style={{
                        padding: `0 ${customPadding}`,
                        backgroundImage: `url(${config.backgroundSecond})`,
                        backgroundSize: "cover",
                        backgroundPosition: "center top",
                        backgroundRepeat: "no-repeat",
                        position: "relative",
                        zIndex: 2,
                    }}
                >
                    <div className="text-center mb-4 max-w-[280px]">
                        <button onClick={handleDownload} id="download-button">
                            <img
                                className="hint-button"
                                src={config.result.outcomeHint}
                                alt="outcome-hint"
                            />
                        </button>
                    </div>
                    <div className="flex gap-[10px] mb-[30px]">
                        <button
                            onClick={handleRestartClick}
                            className="w-[115px]"
                            id="restart-button"
                        >
                            <img
                                className="restart-button"
                                src={config.buttons.restartButton}
                                alt="restart-button-image"
                            />
                        </button>
                        <button
                            onClick={() =>
                                window.open(
                                    "https://www.gtg.org.tw/directsignup?ig=20250412"
                                )
                            }
                            className="w-[115px]"
                            id="rich-god-reserve-button"
                        >
                            <img
                                className="restart-button"
                                src={config.buttons.richGodReserveButton}
                                alt="restart-button-image"
                            />
                        </button>
                        <button
                            onClick={handleShare}
                            className="w-[115px]"
                            id="share-button"
                        >
                            <img
                                className="share-button"
                                src={config.buttons.shareButton}
                                alt="share-to-friends"
                            />
                        </button>
                    </div>

                    <div className="facebook-content w-full max-w-[370px] rounded-3xl">
                        <img src={config.propaganda} alt="propaganda" />
                    </div>

                    <div className="facebook-content  w-full max-w-[370px] rounded-3xl mb-8"></div>

                    <div
                        className="relative facebook-content flex flex-col justify-center items-center 
                        align-center w-full max-w-[370px] rounded-3xl mb-8"
                    >
                        <img src={config.follow} alt="follow" />
                        <div className="absolute top-[35px] flex justify-center space-evenly gap-[45px]">
                            <button
                                className="shop-button"
                                onClick={() =>
                                    window.open(
                                        "https://line.me/R/ti/p/@fao0484g"
                                    )
                                }
                                id="official-line-button"
                            >
                                <img
                                    src={config.buttons.lineButton}
                                    alt="line-logo"
                                    width="45px"
                                />
                            </button>
                            <button
                                className="shop-button"
                                onClick={() =>
                                    window.open(
                                        "https://www.facebook.com/gtg52168"
                                    )
                                }
                                id="official-facebook-button"
                            >
                                <img
                                    src={config.buttons.facebookButton}
                                    alt="facebook-logo"
                                    width="45px"
                                />
                            </button>
                            <button
                                className="shop-button"
                                onClick={() =>
                                    window.open(
                                        "https://www.instagram.com/gtg52168/"
                                    )
                                }
                                id="official-instagram-button"
                            >
                                <img
                                    src={config.buttons.instagramButton}
                                    alt="instagram-logo"
                                    width="45px"
                                />
                            </button>
                        </div>
                    </div>
                    <ConfirmDialog
                        isOpen={showConfirm}
                        onClose={handleCloseConfirm}
                        onConfirm={handleConfirmRestart}
                    />
                </div>
            </div>
        );
    }
);

const ConfirmDialog = memo(({ isOpen, onClose, onConfirm }) => {
    if (!isOpen) return null;

    return (
        <div className="fixed inset-0 bg-black bg-opacity-50 flex items-center justify-center z-50">
            <div className="bg-[#FCDECF] p-6 rounded-lg border-2 border-[#402529] max-w-[300px] w-full mx-4">
                <h3 className="text-xl text-center mb-4">
                    結果將消失，請先儲存
                </h3>
                <div className="flex justify-center gap-4">
                    <button
                        onClick={onConfirm}
                        className="border-1 border:white px-6 py-2 bg-purple-200 hover:bg-purple-300 rounded-lg transition-colors"
                        id="confirm-restart-button"
                    >
                        跳轉
                    </button>
                    <button
                        onClick={onClose}
                        className="px-6 py-2 bg-gray-200 hover:bg-gray-300 rounded-lg transition-colors"
                        id="cancel-restart-button"
                    >
                        取消
                    </button>
                </div>
            </div>
        </div>
    );
});
const Footer = ({ config }) => {
    return (
        <footer className="fixed bottom-0 left-0 w-full bg-black pt-2 pb-2 z-50">
            <div className="max-w-7xl mx-auto">
                <div className="flex items-center justify-center">
                    <div className="flex items-center gap-1">
                        <button
                            className="cursor-pointer"
                            onClick={() => window.open(config.links.SUPERMEDIA)}
                            id="supermedia-logo-button"
                        >
                            <img
                                src={config.superMediaLogoLink}
                                alt="super-media-logo"
                                className="w-[100px]"
                            />
                        </button>
                        <button
                            className="cursor-pointer"
                            onClick={() =>
                                window.open("https://www.facebook.com/gtg52168")
                            }
                            id="temple-logo-button"
                        >
                            <img
                                src={config.templeLogo}
                                alt="temple-logo"
                                className="w-[40px] ml-2 mr-2"
                            />
                        </button>
                        <p className="text-[10px] text-white tracking-tighter">
                            Copyright © 2025. TTSHOW All Rights Reserved.
                        </p>
                    </div>
                </div>
            </div>
        </footer>
    );
};

const LoadingPage = memo(({ config, onLoadingComplete }) => {
    const [loadingProgress, setLoadingProgress] = useState(0);

    useEffect(() => {
        const interval = setInterval(() => {
            setLoadingProgress((prev) => {
                if (prev >= 100) {
                    clearInterval(interval);

                    setTimeout(() => {
                        if (onLoadingComplete) {
                            onLoadingComplete();
                        }
                    }, 300);

                    return 100;
                }
                return prev + 6;
            });
        }, 100);

        return () => clearInterval(interval);
    }, []);

    const totalDistance = 85;
    const currentDistance = (loadingProgress / 100) * totalDistance;

    return (
        <div className="flex w-full min-h-[100dvh] items-center justify-center max-w-[430px]">
            <div className="relative w-full">
                <CardFrame imgType="cloud" config={config} />
                <div className="absolute top-[30px] left-1/2 transform -translate-x-1/2 mb-8">
                    <img
                        src={config.templeLogoLong}
                        alt="廣天宮"
                        className="w-[120px] mx-auto"
                    />
                </div>
                <div
                    className="absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 
                w-full flex flex-col items-center justify-center"
                >
                    <div className="relative w-full max-w-[150px] h-[120px]">
                        <img
                            className="absolute top-[15px] left-[-103px] w-[90px] z-10"
                            src={config.mainVisual}
                            alt="loading-pic"
                            style={{
                                transform: `translateX(${
                                    currentDistance * 3.2
                                }px)`,
                                transition: "transform 0.3s ease-out",
                            }}
                        />
                    </div>

                    <div
                        className="w-full max-w-[280px] h-[28px] border-[2px] border-[#402529]
                                    rounded-[5px] p-[3px] bg-[#F7E8CE] overflow-hidden mb-4"
                    >
                        <div
                            className="h-full bg-[#AC2409] transition-all duration-300 ease-out 
                            border-[2px] border-[#402529] "
                            style={{ width: `${loadingProgress}%` }}
                        />
                    </div>

                    <p className="text-[#402529] font-bold">
                        {/* {loadingProgress}% */}
                        <div className="h-8 mb-4 text-center">
                            <img
                                className="w-[90px] z-10"
                                src={config.loadingPic}
                                alt="loading-pic"
                            />
                        </div>
                    </p>
                </div>
            </div>
        </div>
    );
});

const App = () => {
    const [staticJson, setStaticJson] = useState(STORAGE_DATA);
    const QUESTIONS = staticJson.config.questions;

    const [isInitializing, setIsInitializing] = useState(true);

    const [currentQuestionIndex, setCurrentQuestionIndex] = useState(-1);
    const [scores, setScores] = useState([]);
    const [totalScore, setTotalScore] = useState(0);
    const [resultImage, setResultImage] = useState("");
    const [isMobile, setIsMobile] = useState(false);

    const [isLoading, setIsLoading] = useState(false);
    const [isAnimationComplete, setIsAnimationComplete] = useState(false);

    const [question10Choice, setQuestion10Choice] = useState("");

    const [resultType, setResultType] = useState("");

    useEffect(() => {
        const preloadImages = async () => {
            try {
                const imagesToPreload = [
                    staticJson.config.cardStart,
                    staticJson.config.realBackgroundCloud,
                    staticJson.config.mainVisual,
                    staticJson.config.startHint,
                    staticJson.config.buttons.startButton,
                ];

                const imagePromises = imagesToPreload.map((src) => {
                    return new Promise((resolve, reject) => {
                        const img = new Image();
                        img.onload = () => resolve(img);
                        img.onerror = () => resolve(null);
                        img.src = src;
                    });
                });

                await Promise.allSettled(imagePromises);
            } catch (err) {
                console.error("Image preloading error:", err);
            } finally {
                setIsInitializing(false);
            }
        };

        preloadImages();
    }, []);

    const handleBack = useCallback(() => {
        setIsLoading(true);
        try {
            if (currentQuestionIndex === 0) {
                setCurrentQuestionIndex(-1);
                setScores([]);
                setTotalScore(0);
                return;
            }
            const prevIndex = currentQuestionIndex - 1;

            if (currentQuestionIndex === 9) {
                setQuestion10Choice("");
            }
            const newScores = [...scores];
            const removedScore = newScores.pop() || 0;
            setScores(newScores);
            setTotalScore((prevScore) => prevScore - removedScore);

            setCurrentQuestionIndex(prevIndex);
        } finally {
            setIsLoading(false);
        }
    }, [currentQuestionIndex, scores]);

    const handleRestart = useCallback(async () => {
        setIsLoading(true);
        try {
            setCurrentQuestionIndex(-1);
            setScores([]);
            setTotalScore(0);
            setResultImage("");
            setQuestion10Choice("");
            setResultType("");
        } finally {
            setIsLoading(false);
        }
    }, []);

    const handleStart = async () => {
        setIsLoading(true);
        try {
            setCurrentQuestionIndex(0);
            setScores([]);
            setTotalScore(0);
            setResultImage("");
            setQuestion10Choice("");
        } finally {
            setIsLoading(false);
        }
    };

    useEffect(() => {
        const checkDevice = () => {
            setIsMobile(window.innerWidth <= 768);
        };

        console.log("check device");
        checkDevice();
        window.addEventListener("resize", checkDevice);

        return () => window.removeEventListener("resize", checkDevice);
    }, []);

    const determineResultImage = useCallback((score) => {
        if (score >= 76 && score <= 90) {
            return staticJson.config.resultVisual.top;
        } else if (score >= 66 && score <= 75) {
            return staticJson.config.resultVisual.second;
        } else if (score >= 56 && score <= 65) {
            return staticJson.config.resultVisual.third;
        } else if (score >= 46 && score <= 55) {
            return staticJson.config.resultVisual.fourth;
        } else if (score >= 36 && score <= 45) {
            return staticJson.config.resultVisual.fifth;
        } else {
            return staticJson.config.resultVisual.last;
        }
    }, []);

    const currentQuestionData = useMemo(() => {
        if (
            currentQuestionIndex < 0 ||
            currentQuestionIndex >= QUESTIONS.length
        ) {
            return null;
        }
        const question = QUESTIONS[currentQuestionIndex];
        return {
            ...question,
        };
    }, [currentQuestionIndex, QUESTIONS]);

    const handleAnswer = useCallback(
        async (score, choiceLetter) => {
            setIsAnimationComplete(false);
            try {
                const newScores = [...scores, score];
                const newTotalScore = totalScore + score;
                setScores(newScores);
                setTotalScore(newTotalScore);

                if (currentQuestionIndex === 9 && choiceLetter) {
                    console.log("第10題選擇：", choiceLetter);
                    setQuestion10Choice(choiceLetter);
                }

                if (currentQuestionIndex === QUESTIONS.length - 1) {
                    setIsLoading(true);
                    setIsAnimationComplete(false);

                    await new Promise((resolve) => setTimeout(resolve, 1000));

                    const resultImg = determineResultImage(newTotalScore);
                    setResultImage(resultImg);

                    let type = "";
                    if (newTotalScore >= 76 && newTotalScore <= 90) {
                        type = "top-天財";
                    } else if (newTotalScore >= 66 && newTotalScore <= 75) {
                        type = "second-將財";
                    } else if (newTotalScore >= 56 && newTotalScore <= 65) {
                        type = "third-秀財";
                    } else if (newTotalScore >= 46 && newTotalScore <= 55) {
                        type = "fourth-庸財";
                    } else if (newTotalScore >= 36 && newTotalScore <= 45) {
                        type = "fifth-蠢財";
                    } else {
                        type = "last-奴財";
                    }

                    setResultType(type);
                    setCurrentQuestionIndex(currentQuestionIndex + 1);
                } else {
                    setCurrentQuestionIndex(currentQuestionIndex + 1);
                }
            } catch (error) {
                setIsLoading(false);
                console.log(error);
            }
        },
        [
            currentQuestionIndex,
            scores,
            totalScore,
            QUESTIONS.length,
            determineResultImage,
        ]
    );

    const handleAnimationComplete = useCallback(() => {
        setIsAnimationComplete(true);

        if (isLoading) {
            setIsLoading(false);
        }
    }, [isLoading]);

    if (isInitializing) {
        return (
            <div className="flex w-full min-h-[100dvh] items-center justify-center">
                <div className="text-center">
                    <div className="w-16 h-16 border-4 border-[#AC2409] border-t-transparent rounded-full animate-spin mx-auto mb-4"></div>
                    <p className="text-[#402529]">載入中...</p>
                </div>
            </div>
        );
    }

    if (isLoading) {
        return (
            <LoadingPage
                config={staticJson.config}
                onLoadingComplete={handleAnimationComplete}
            />
        );
    }

    if (currentQuestionIndex === -1) {
        return (
            <StartPage
                config={staticJson.config}
                onStart={handleStart}
                currentQuestionIndex={currentQuestionIndex}
            />
        );
    }

    if (currentQuestionIndex < QUESTIONS.length && currentQuestionIndex >= 0) {
        return (
            <QuestionPage
                currentQuestion={currentQuestionData}
                onAnswer={handleAnswer}
                config={staticJson.config}
                onBack={handleBack}
                currentScore={totalScore}
            />
        );
    }

    console.log("結果頁面 - 第10題選擇:", question10Choice);

    return (
        <div className="flex flex-col w-full">
            <main className="flex flex-col items-center">
                <section className="w-full text-center">
                    <ResultPage
                        resultImage={
                            resultImage || staticJson.config.resultVisual.top
                        }
                        totalScore={totalScore}
                        config={staticJson.config}
                        onRestart={handleRestart}
                        question10Choice={question10Choice}
                        resultType={resultType}
                    />
                    <Footer config={staticJson.config} />
                </section>
            </main>
        </div>
    );
};

ReactDOM.render(<App />, document.querySelector("#root"));
